#!/bin/sh

#
# "SystemImager" - Copyright (C) 1999-2001 Brian Elliott Finley <brian@systemimager.org>
#
#
# This master autoinstall script was created with SystemImager v2.0.1

VERSION=2.0.1
IMAGENAME=test
PATH=/sbin:/bin:/usr/bin:/usr/sbin:/tmp
ARCH=`uname -m \
| sed -e s/i.86/i386/ -e s/sun4u/sparc64/ -e s/arm.*/arm/ -e s/sa110/arm/`

shellout() {
  exec cat /etc/issue ; exit 1
}

### BEGIN Check to be sure this not run from a working machine ###
# test for mounted SCSI or IDE disks
mount | grep [hs]d[a-z][1-9] > /dev/null 2>&1
[ $? -eq 0 ] &&  echo Sorry.  Must not run on a working machine... && shellout

# test for mounted software RAID devices
mount | grep md[0-9] > /dev/null 2>&1
[ $? -eq 0 ] &&  echo Sorry.  Must not run on a working machine... && shellout

# test for mounted hardware RAID disks
mount | grep c[0-9]+d[0-9]+p > /dev/null 2>&1
[ $? -eq 0 ] &&  echo Sorry.  Must not run on a working machine... && shellout
### END Check to be sure this not run from a working machine ###

# Pull in variables left behind by the linuxrc script.
# This information is passed from the linuxrc script on the autoinstall media via /tmp/variables.txt
# Apparently the shell on the autoinstall media is not intelligent enough to take a "set -a" parameter
. /tmp/variables.txt || shellout

### BEGIN Stop RAID devices before partitioning begins ###
# Why did they get started in the first place?  So we can pull a local.cfg
# file off a root mounted RAID system.  We do this even if a system does not
# use RAID, in case you are using a disk that was previously part of a RAID
# array.

# find running raid devices
RAID_DEVICES=`cat /proc/mdstat | grep ^md | mawk '{print "/dev/" $1}'`

# get raidstop utility if any raid devices exist
if [ ! -z "${RAID_DEVICES}" ]
then
  rsync -av --numeric-ids $IMAGESERVER::${ARCH}-boot/raidstop /tmp/ || shellout
fi

# raidstop will not run unless a raidtab file exists
touch /etc/raidtab || shellout

# turn dem pesky raid devices off!
for RAID_DEVICE in ${RAID_DEVICES}
do
  # we don't do a shellout here because, well I forgot why, but we don't.
  echo "raidstop ${RAID_DEVICE}" && raidstop ${RAID_DEVICE}
done
### END Stop RAID devices before partitioning begins ###

# Partition the disk (see the sfdisk man page for customization details)
#
# Below is an example of how to customize a disk.  This can be useful if your clients
# have disks of differing sizes or geometries.  Here is a description of the sfdisk
# command presented in the example below:
#
# line 1 -- the sfdisk command and arguments (-uM means "units are MB)
# line 2 -- this is a comment
# line 3 -- start at the beginning of the disk 0, give 22M /dev/sda1 (/boot)
# line 4 -- start where the last partition left off, and add 512M /dev/sda2 for swap, partition type 82
# line 5 -- start where the last partition left off, and add 6000M /dev/sda3 (/)
# line 6 -- start where the last partition left off, and give the remaining space to /dev/sda4 (/var)
# line 7 -- the EOF statement indicating to sfdisk that we are finished giving it commands
#
# <<<< BEGIN EXAMPLE >>>>
#sfdisk -uM /dev/sda <<EOF
## partition    start_of_partition      size_in_megabytes       partition_type
#  /dev/sda1 :  start= 0,          	size= 22,               Id=83
#  /dev/sda2 :  start= ,                size= 512,              Id=82
#  /dev/sda3 :  start= ,                size= 6000,             Id=83
#  /dev/sda4 :  start= ,                size= ,                 Id=83
#EOF
# <<<< END EXAMPLE >>>>

# partition hda
echo partitioning hda...
sleep 1s
sfdisk -L -uM /dev/hda <<EOF || shellout
/dev/hda1 : start= 0, size= 62, Id=83
/dev/hda2 : start= , size= 8189, Id=83
/dev/hda3 : start= , size= 4094, Id=83
/dev/hda4 : start= , size= , Id=f
/dev/hda5 : start= , size= 2047, Id=82
/dev/hda6 : start= , size= , Id=83
EOF

# initialize swap devices
mkswap -v1 /dev/hda5 || shellout
swapon /dev/hda5 || shellout

# get necessary files so that mkfs.xfs will work
rsync -av --numeric-ids $IMAGESERVER::${ARCH}-boot/mkfs.xfs /tmp/ || shellout
# format xfs devices
mkfs.xfs -f /dev/hda1 || shellout
mkfs.xfs -f /dev/hda2 || shellout
mkfs.xfs -f /dev/hda3 || shellout
mkfs.xfs -f /dev/hda6 || shellout
# create mount points and mount filesystems
mkdir -p /a/ || shellout
mount /dev/hda6 /a/ -t xfs -o defaults || shellout

mkdir -p /a/boot || shellout
mount /dev/hda1 /a/boot -t xfs -o defaults || shellout

mkdir -p /a/proc || shellout
mount proc /a/proc -t proc -o defaults || shellout

mkdir -p /a/tmp || shellout
mount /dev/hda2 /a/tmp -t xfs -o defaults || shellout

mkdir -p /a/usr || shellout
mount /dev/hda3 /a/usr -t xfs -o defaults || shellout

# Filler up!
#
# If we are installing over ssh, we must limit the bandwidth used by 
# rsync with the --bwlimit option.  This is because of a bug in ssh that
# causes a deadlock.  The only problem with --bwlimit is that it slows 
# down your autoinstall significantly.  We try to guess which one you need:
# o if you ran getimage with -ssh-user, we presume you need --bwlimit
# o if you ran getimage without -ssh-user, we presume you don't need 
#   --bwlimit and would rather have a faster autoinstall.
#
# Both options are here for your convenience.  We have done our best to 
# choose the one you need and have commented out the other.
#
#rsync -av --bwlimit=10000 --numeric-ids $IMAGESERVER::$IMAGENAME/ /a/ || shellout

rsync -av --numeric-ids $IMAGESERVER::$IMAGENAME/ /a/ || shellout

# Leave notice of which image is installed on the client
echo $IMAGENAME > /a/etc/systemimager/IMAGE_LAST_SYNCED_TO || shellout

chroot /a/ systemconfigurator --configsi --stdin <<EOL || shellout

[INTERFACE0]
DEVICE = eth0
TYPE = dhcp

EOL

# unmount the freshly filled filesystems
echo -n umount /a/usr/ ... 
umount /a/usr/ && echo Done! || shellout
echo -n umount /a/tmp/ ... 
umount /a/tmp/ && echo Done! || shellout
echo -n umount /a/proc/ ... 
umount /a/proc/ && echo Done! || shellout
echo -n umount /a/boot/ ... 
umount /a/boot/ && echo Done! || shellout
echo -n umount /a// ... 
umount /a// && echo Done! || shellout

# No need to manually stop software RAID devices -- the autodetect support
# in the kernel will do that for us.

# Take network interface down
ifconfig eth0 down || shellout

# Cause the system to make noise and display an "I'm done." message
ralph="sick"
count="1"
while [ $ralph="sick" ]
do
  beep
  [ $count -lt 60 ] && echo "I've been done for $count seconds.  Reboot me already!"
  [ $(($count / 60 * 60)) = $count ] && echo "I've been done for $(($count / 60)) minutes now.  Reboot me already!"
  sleep 1
  count=$(($count + 1))
done

